#!/bin/bash

function get_property {
    FILE_PATH=$1
    PROPERTY_KEY=$2
    # output property file | find line with provided key | remove key with equal sign to get <VALUE> | replace all '\=' to '=' | remove comment at the end of line | remove '\r' | remove '\n'
    PROPERTY_VALUE=$(cat "$FILE_PATH" | grep "^ *$PROPERTY_KEY *=" | sed 's/^[^=]*=//' | sed 's/\\=/=/g' | sed 's/\(.*\)#.*/\1/' | tr -d '\r' | tr -d '\n')
    echo ${PROPERTY_VALUE}
}

function abs_path() {
    # получение абсолютного пути из относительного
    if [ -d "$1" ]; then
        # dir
        (cd "$1"; pwd)
    elif [ -f "$1" ]; then
        # file
        if [[ $1 = /* ]]; then
            echo "$1"
        elif [[ $1 == */* ]]; then
            echo "$(cd "${1%/*}"; pwd)/${1##*/}"
        else
            echo "$(pwd)/$1"
        fi
    fi
}

function run_java_application {
    WORKING_DIR=$1
    SCRIPT_NAME=$2
    STARTER_CONTEXT=$3

    # store original java home and classpath
    ORIGINAL_JAVA_HOME=${JAVA_HOME}
    ORIGINAL_CLASSPATH=${CLASSPATH}

    CONFIG_PROPERTIES="bin/$SCRIPT_NAME.properties"
    CONFIG="$WORKING_DIR/$CONFIG_PROPERTIES"
    # read config properties
    if [ ! -f "$CONFIG" ]
    then
        echo "$CONFIG not found"
        exit
    fi

    JAVA_HOME=$(get_property "$CONFIG" "java.home")
    JAVA_XMS=$(get_property "$CONFIG" "java.xms")
    JAVA_XMX=$(get_property "$CONFIG" "java.xmx")
    JAVA_XMN=$(get_property "$CONFIG" "java.xmn")
    JAVA_XSS=$(get_property "$CONFIG" "java.xss")

    APP_MAIN_CLASS=$(get_property "$CONFIG" "app.main.class")
    APP_SHOW_CONSOLE=$(get_property "$CONFIG" "app.show.console")
    unset APP_PARAMS

    if [ ! -z "$STARTER_CONTEXT" ]
    then
        APP_PARAMS=$STARTER_CONTEXT
    fi

    if [ -z "$JAVA_HOME" ]
    then
        echo "java.home is empty"
        exit
    fi

    pushd "$WORKING_DIR" > /dev/null

    if [ -f "$JAVA_HOME/bin/java" ]
    then
        JAVA=$(abs_path "$JAVA_HOME/bin/java")
    else
        echo "$JAVA_HOME/bin/java not found"
        exit
    fi

    CLASSPATH=.:./*:./lib/*:./addins/*:./bin
    JAVA_OPTIONS="-showversion -Xms$JAVA_XMS -Xmx$JAVA_XMX -Xmn$JAVA_XMN -Xss$JAVA_XSS -Dfile.encoding=UTF-8 -splash:bin/$SCRIPT_NAME.opss"
    COMMAND="$JAVA_OPTIONS -classpath $CLASSPATH $APP_MAIN_CLASS"

    # без этого переменная окружения HOSTNAME недоступна из Java
    export HOSTNAME

    CUR_APP_ID=$(ps -o 'ppid=' -p $$ | xargs)
    while true
    do
        CUR_APP="$(ps -o 'cmd=' -p $CUR_APP_ID)"
        CUR_APP="${CUR_APP%% *}"
        if [[ "$CUR_APP" != *bash ]]
        then
            break
        else
            CUR_APP_ID=$(ps -o 'ppid=' -p ${CUR_APP_ID} | xargs)
        fi
    done

    if [ "$APP_SHOW_CONSOLE" == "true" ]
    then
        echo "Command: $JAVA $COMMAND $APP_PARAMS"
        nohup "$CUR_APP" -e "\"$JAVA\" ${COMMAND} \"$APP_PARAMS\" -t=$CUR_APP" &>/dev/null &
        sleep 1
    else
        nohup "$CUR_APP" -e "bash -c 'nohup \"$JAVA\" ${COMMAND} \"$APP_PARAMS\" -t=$CUR_APP -tid=$BASHPID,$CUR_APP_ID &>/dev/null & sleep 30'" &>/dev/null &
        sleep 1
    fi

    popd > /dev/null

    # restore original java home and classpath
    CLASSPATH=${ORIGINAL_CLASSPATH}
    JAVA_HOME=${ORIGINAL_JAVA_HOME}
}

function run_java_service_application {
    WORKING_DIR=$1
    CONFIG_PROPERTIES=$2
    STARTER_CONTEXT=$3

    # store original java home and classpath
    ORIGINAL_JAVA_HOME=${JAVA_HOME}
    ORIGINAL_CLASSPATH=${CLASSPATH}

    CONFIG="$WORKING_DIR/$CONFIG_PROPERTIES"
    # read config properties
    if [ ! -f "$CONFIG" ]
    then
        echo "$CONFIG not found"
        exit
    fi

    JAVA_HOME=$(get_property "$CONFIG" "java.home")
    JAVA_XMS=$(get_property "$CONFIG" "java.xms")
    JAVA_XMX=$(get_property "$CONFIG" "java.xmx")

    APP_MAIN_CLASS=$(get_property "$CONFIG" "app.main.class")
    APP_SHOW_CONSOLE=$(get_property "$CONFIG" "app.show.console")
    unset APP_PARAMS

    if [ ! -z "$STARTER_CONTEXT" ]
    then
        APP_PARAMS=$STARTER_CONTEXT
    fi

    if [ -z "$JAVA_HOME" ]
    then
        echo "java.home is empty"
        exit
    fi

    pushd "$WORKING_DIR" > /dev/null

    if [ -f "$JAVA_HOME/bin/java" ]
    then
        JAVA=$(abs_path "$JAVA_HOME/bin/java")
    else
        echo "$JAVA_HOME/bin/java not found"
        exit
    fi

    CLASSPATH=.:./*:./lib/*:./addins/*:./bin
    JAVA_OPTIONS="-showversion -Xms$JAVA_XMS -Xmx$JAVA_XMX"
    COMMAND="$JAVA_OPTIONS -classpath $CLASSPATH $APP_MAIN_CLASS"

    # без этого переменная окружения HOSTNAME недоступна из Java
    export HOSTNAME

    CUR_APP_ID=$(ps -o 'ppid=' -p $$ | xargs)
    while true
    do
        CUR_APP="$(ps -o 'cmd=' -p $CUR_APP_ID)"
        CUR_APP="${CUR_APP%% *}"
        if [[ "$CUR_APP" != *bash ]]
        then
            break
        else
            CUR_APP_ID=$(ps -o 'ppid=' -p ${CUR_APP_ID} | xargs)
        fi
    done

    if [ "$APP_SHOW_CONSOLE" == "true" ]
    then
        echo "Command: $JAVA $COMMAND $APP_PARAMS"
        nohup "$CUR_APP" -e "\"$JAVA\" ${COMMAND} \"$APP_PARAMS\" -t=$CUR_APP" &>/dev/null &
        sleep 1
    else
        nohup "$CUR_APP" -e "bash -c 'nohup \"$JAVA\" ${COMMAND} \"$APP_PARAMS\" -t=$CUR_APP -tid=$BASHPID,$CUR_APP_ID &>/dev/null & sleep 30'" &>/dev/null &
        sleep 1
    fi

    popd > /dev/null

    # restore original java home and classpath
    CLASSPATH=${ORIGINAL_CLASSPATH}
    JAVA_HOME=${ORIGINAL_JAVA_HOME}
}
